<?php

namespace CADSUP\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class ConsultaCNPJ
{

    protected $environment;
    protected $connection;
    protected $entityManager;

    public function __construct($entityManager, $environment)
    {
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function consultaCNPJ(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $NrInscricao = $row['NRINSJURFORN'];
        $TipoInscricao = $row['IDTPIJURFORN'];
        $client = new \SoapClient('http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx?WSDL');

        if ($TipoInscricao == 'J')
            $function = 'PessoaJuridicaNFe';

        $arguments = array(array(
            'Credenciais' => array(
                'Email' => 'administrativo@teknisa.com',
                'Senha' => 'K1*#n2'
            ),
            'Documento'    => $NrInscricao
        ));
        $options = array('location' => 'http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx');

        try {
            $data = $client->__soapCall($function, $arguments, $options);
            $data = $this->obj_to_array($data);

            if ($data['PessoaJuridicaNFeResult']['Transacao']['Status'] !== false && isset($data['PessoaJuridicaNFeResult']['Enderecos'])) {
                $retornoSelect = $this->buscaCDmunicipio($data);

                if ($retornoSelect && count($retornoSelect) > 0) {
                    $data['SGESTADO'] = $retornoSelect["SGESTADO"];
                    $data['CDMUNICIPIO'] = $retornoSelect["CDMUNICIPIO"];
                }
            }

            $dataSet = new DataSet("data", array($data));
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            if ($data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] == 'Falha na transacao') {
                $data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] = 'Não foi possível realizar a consulta no momento. Tente Novamente. Caso o problema persista, entre em contato com o Suporte.';
            } else if ($data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] == 'Documento invalido para consulta') {
                $data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] = 'Número de Inscrição inválido para consulta. Tente Novamente. Caso o problema persista, entre em contato com o Suporte.';
            } else if ($data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] == 'Credenciais de Acesso (Usuario e/ou Senha) Invalidos') {
                $data['PessoaJuridicaNFeResult']['Transacao']['CodigoStatusDescricao'] = 'Credenciais de Acesso (Usuário e/ou Senha) Inválidos';
            }

            return  array("Status" => false, "CONEXAO" => true);
        }
    }

    public function buscaCDmunicipio($data)
    {
        $sql = "SELECT CDMUNICIPIO, SGESTADO
                FROM MUNICIPIO
              WHERE CDPAIS = '0055'
                AND SGESTADO = :SGESTADO
                AND (UPPER(NMMUNICIPIO) = UPPER(:NMMUNICIPIO)
                  OR UPPER(NMMUNICIPIO) = UPPER(:NMMUNICIPIO_SEMACENTO))";
        $params['SGESTADO'] = $data['PessoaJuridicaNFeResult']['Enderecos']['Estado'];
        $params['NMMUNICIPIO'] = $data['PessoaJuridicaNFeResult']['Enderecos']['Cidade'];
        $params['NMMUNICIPIO_SEMACENTO'] = $this->semAcento($data['PessoaJuridicaNFeResult']['Enderecos']['Cidade']);

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $dataSet = $std->fetch();
    }

    public function semAcento($texto)
    {
        $acentos = array(
            'á' => 'a',
            'à' => 'a',
            'ã' => 'a',
            'â' => 'a',
            'ä' => 'a',
            'é' => 'e',
            'ê' => 'e',
            'è' => 'e',
            'ë' => 'e',
            'í' => 'i',
            'ì' => 'i',
            'ï' => 'i',
            'î' => 'i',
            'ó' => 'o',
            'ô' => 'o',
            'ò' => 'o',
            'õ' => 'o',
            'ö' => 'o',
            'ú' => 'u',
            'ü' => 'u',
            'ù' => 'u',
            'û' => 'u',
            'ç' => 'c',
            'Á' => 'A',
            'À' => 'A',
            'Ã' => 'A',
            'Â' => 'A',
            'Ä' => 'A',
            'É' => 'E',
            'Ê' => 'E',
            'È' => 'E',
            'Ë' => 'E',
            'Í' => 'I',
            'Ì' => 'I',
            'Ï' => 'I',
            'Î' => 'I',
            'Ó' => 'O',
            'Ô' => 'O',
            'Õ' => 'O',
            'Ò' => 'O',
            'Ö' => 'O',
            'Ú' => 'U',
            'Ù' => 'U',
            'Û' => 'U',
            'Ü' => 'U',
            'Ç' => 'C'
        );

        return $texto = strtr($texto, $acentos);
    }

    public function obj_to_array($data)
    {
        if (is_array($data) || is_object($data)) {
            $result = array();
            foreach ($data as $key => $value) {
                $result[$key] = $this->obj_to_array($value);
            }
            return $result;
        }
        return $data;
    }
}
