<?php
/**
 * Fechamento.
 *
 * @package Logic
 *
 * @author  alessandrobartels
 */
class Fechamento{

    private $_nrorg;
    private $_nr_tipo_calculo;
    private $_mes_competencia;

    private $_itens_calculo;
    private $_itens_periodo_seguinte;
    private $_eventom;
    private $_itens_decimo_terceiro;
    private $_valorCredito13;	
	


    /**
     * Construtor.
     *
     * @param integer    $nrorg           Numero da Organizacao.
     * @param integer    $nr_tipo_calculo Numero tipo de Calculo.
     * @param TkDateTime $mes_competencia Mes de Competencia.
     *
     * @author alessandrobartels
     */
    public function __construct($nrorg, $nr_tipo_calculo, TkDateTime $mes_competencia){

        $this->_nrorg = $nrorg;
        $this->_nr_tipo_calculo = $nr_tipo_calculo;
        $this->_mes_competencia = $mes_competencia;

        $this->_itens_periodo_seguinte = new FpaitecalcfolhaEntity;
        $this->_itens_decimo_terceiro  = new FpaitecalcfolhaEntity;
        $this->_valorCredito13         = 0;				

        /* Container */
        $this->_eventom = TkDI::getContainer()->eventom;
    }


    /**
     * Fechamento do movimento mensal do calculo.
     *
     * @param GpevinculomEntity     $vinculo         Instancia de GpevinculomEntity.
     * @param FpaitecalcfolhaEntity $itensCalculo    Instancia de FpaitecalcfolhaEntity.
     * @param FpatpcalcmovEntity    $tp_calc_mov_seg Instancia de FpatpcalcmovEntity.
     *
     * @throws Exception E.
     *
     * @return void
     */
    public function fechar(
        GpevinculomEntity $vinculo, FpaitecalcfolhaEntity $itensCalculo, FpatpcalcmovEntity $tp_calc_mov_seg
    ) {

        if ($tp_calc_mov_seg->retornaItensCalculo($vinculo) === false) {
            return;
        }

        $this->_itens_calculo = $itensCalculo;

        try{
            $obj_tp_calc_mov_seg = current(
                $tp_calc_mov_seg->retornaPosicaoTpMov($tp_calc_mov_seg->retornaTpMovPrincipal(), 1)
            );
            $this->_itens_periodo_seguinte =  $obj_tp_calc_mov_seg->getInstancia();
			
            //Marcelo 10/02/2015 0000155865 Fechamento de Folha não deve interferir em rescisões e cálculos já gravados
            $itens = $this->_itens_periodo_seguinte->getItems();
            foreach ($itens as $item) {
                if ((FpatipoassoceveEntity::ASSOC_LIQUIDO == $item->getNreventom()) or (FpatipoassoceveEntity::ASSOC_LIQUIDO_RESCISAO == $item->getNreventom())){
                    return;
                }
            }					
        } catch(Exception $e){
            $message = $e->getMessage().' - Principal FECHAMENTO.PHP';
            throw new Exception($message);
        }

        //Marcelo 09/02/2015 Arredondamento de credito do 13º salario ser gerado como debito para a folha do mes
        //da competência seguinte
        try{
            $obj_tp_calc_13 = current($tp_calc_mov_seg->retornaPosicaoTpMov(FpatipomovimentEntity::TP_MOV_13SAL));
            $this->_itens_decimo_terceiro =  $obj_tp_calc_13->getInstancia();
            $itens = $this->_itens_decimo_terceiro->getItems();

            foreach ($itens as $item) {
                if (FpatipoassoceveEntity::ASSOC_ARREDONDAM_SAL_CR == $item->getNreventom()) {
                    $this->_valorCredito13 = $item->getVreventofolha();
                }
            }

        } catch(Exception $e){
            $message = $e->getMessage().' - 13º SALARIO FECHAMENTO.PHP';
            throw new Exception($message);
        }		

        // Fechar item calc.
        $this->projetaPeriodoSeguinte();

        $mes_competencia = new TkDateTime($this->_mes_competencia, TkDateTime::FORMAT_DATE);

//        $num_mes = (int)$mes_competencia->getMonth()->toValue(TkDateTime::MONTH);
        $num_mes = $mes_competencia->toValue(TkDateTime::MONTH);			

        if ($num_mes === 12) {
            $this->projetaVinculoFimAno($vinculo->historico);
        }

        /*
        // O fechamento retorna o item calculo projeta os itens e persist o itens_periodo_seguinte.
          $items = $this->_itens_periodo_seguinte->getItems();
        //@todo Rever o persist
           if (count($items) > 0) {
          //  $this->_itens_periodo_seguinte->persist();
       	}
        */
    }


    /**
     * Projeta o historico do vinculo para o primeiro dia do ano seguinte ao mes de competencia.
     *
     * @param GpevinculohEntity $vinculoh Vinculo Historico.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function projetaVinculoFimAno(GpevinculohEntity $vinculoh) {

        // Projeta o mes para janeiro do ano seguinte.
        $mes_competencia_ano_seguinte = new TkDateTime($this->_mes_competencia, TkDateTime::FORMAT_DATE);

		
        // Seta o primeiro dia do ano seguinte ao mes de competencia.
        $mes_competencia_ano_seguinte->setDay(1)->setMonth(1);
        $mes_competencia_ano_seguinte->addYear(1);

        //$vinculo =	$vinculoh->projetaVinculo($vinculoh, $mes_competencia_ano_seguinte);
        $vinculo =	$vinculoh->projetaVinculo($mes_competencia_ano_seguinte);

        // Atualiza contribuicao sindical.
        if ($vinculo->getIdcontribuisind() === GpevinculohEntityBase::IDCONTRIBUISIND_DESCONTOU) {
            $vinculo->setIdcontribuisind(GpevinculohEntityBase::IDCONTRIBUISIND_SEMPRE);
        }

        $vinculo->persist();

        unset($mes_competencia_ano_seguinte);
        unset($vinculo);
    }


    /**
     * Projeta um novo Calculo de folha com data de ocorrencia no mes posterior ao mes de competencia.
     *
     * Condicoes para projecao: Ocorrencia do item de calculo ser > 1 (numero de parcelas do evento).
     * 							Data de ocorrencia do item de calculo > mes de competencia.
     * 							Tipo de ocorrencia do evento ser FIXA.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function projetaPeriodoSeguinte(){

        $itens = $this->_itens_calculo->getItems();

        $novo_mes_ocorrencia = new TkDateTime();
        $mes_competencia_aux = new TkDateTime();

        foreach ($itens as $item) {
            $projeta = false;
            $evento = $this->_eventom->retornaEvento($item->getNreventom());
            $novo_contador = $item->getQtocorproxperi();
            if ($novo_contador > 1) {
                $projeta = true;
                $novo_contador--;
            }

            $novo_mes_ocorrencia->set($item->getDtocorrencia(), TkDateTime::FORMAT_DATE);
            $mes_competencia_aux->set($item->getDtocorrencia(), TkDateTime::FORMAT_DATE);

            // Volta a data para o dia primeiro. Desta forma a comparacao eh feita pelo mes/ano.
            $mes_competencia_aux->setDay(1);
			
            if ($mes_competencia_aux->isLater($this->_mes_competencia, TkDateTime::FORMAT_DATE) === true) {
                $nr_novo_evento = $evento->historico->getNreventotransf();
                if (empty($nr_novo_evento) === false) {
                    $projeta = true;
                }
            } else {
                // Se nao foi o mes que provocou a projecao eh preciso projetar o mes seguinte.
                $novo_mes_ocorrencia->addMonth(1);				
				//18-11-2014 Marcelo - O evento de Complemento 760 Complemento de Líquido Negativo nao estava sendo projetado
				//porque não havia o teste para passar o valor true para a variavel $projeta (somente adicionava-se 1 mes a competencia)
                $nr_novo_evento = $evento->historico->getNreventotransf();
                if (empty($nr_novo_evento) === false) {
                    $projeta = true;
                }			  				 
            }

            // @todo verificar se so projeta tipo de ocorrencia FIXA.
            if ($evento->historico->getIdtipoocorreven() === FpaeventohEntity::IDTIPOOCORREVEN_FIXA) {
                $projeta = true;
            }

            if ($projeta === true) {
                $nr_novo_evento = $evento->historico->getNreventotransf();
                if (empty($nr_novo_evento) === true) {
                    $novo_evento = $evento;
                } else {
                    $novo_evento = $this->_eventom->retornaEvento($nr_novo_evento);
                }

                if ($evento->historico->getIdzeraevenfecha() === 'S') {
                    $novo_valor = 0;
                } else {
				
					//Marcelo 09/02/2015 Arredondamento de credito do 13º salario ser gerado como debito para a folha do mes
					//da competência seguinte				
                    if (FpatipoassoceveEntity::ASSOC_ARREDONDAM_SAL_CR == $evento->historico->getNreventom()) {
                        if ($this->_valorCredito13 > 0) {					
                            $novo_valor = $item->getVreventofolha() + $this->_valorCredito13;
                        } else {
                            $novo_valor = $item->getVreventofolha();
                        }
                    }	else {
                    //BORBA 15/05/2015 Modificação acima garantia apenas que ASSOC_ARREDONDAM_SAL_CR fosse projetado com valor
                        $novo_valor = $item->getVreventofolha();
                    }
                }

                $nova_referencia = $item->getVrreffolha();
                $this->_itens_periodo_seguinte->geraEvento(
                    $novo_evento, $novo_mes_ocorrencia, $novo_valor, $nova_referencia, $novo_contador
                );
            }
        }

    }


}